/*
 * Copyright (C) 2006 Johannes Baumgarten, Bjoern Gebhardt, Mathias Kellner, Stephan Kluge, Svetlana Li, Jurij Merker, Daniel Meyer, Andre Schucht, Ling Wu
 *
 * This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 2 of the License. 
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details. 
 * You should have received a copy of the GNU General Public License along with this program; if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA
 */

package Debug.Visualizer.editparts;

import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeEvent;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.LayeredPane;
import org.eclipse.draw2d.XYLayout;
import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.draw2d.geometry.Rectangle;

import org.eclipse.gef.EditPart;
import org.eclipse.gef.EditPolicy;
import org.eclipse.gef.GraphicalEditPart;
import org.eclipse.gef.editparts.AbstractGraphicalEditPart;

import Debug.Visualizer.model.DiagramElementModel;
import Debug.Visualizer.model.DiagramModel;
import Debug.Visualizer.model.HostModel;
import Debug.Visualizer.model.InfoscreenModel;

import Debug.Visualizer.editpolicies.MoveableEditPolicy;

/**
 * Controls the the stored Data in the DiagramModel and the Parent Layout
 * specified in createFigure
 * 
 * @author Daniel Meyer
 */
public class DiagramEditPart extends AbstractGraphicalEditPart implements
        PropertyChangeListener {

    @SuppressWarnings("unused")
    private List<Point> pointlist = new ArrayList<Point>();

    /**
     * Creates a org.eclipse.draw2d.IFigure here a LayeredPane() and a Layout
     * 
     * @return IFigure
     */
    @Override
    protected IFigure createFigure() {

    	  //create figure
        IFigure figure1 = new LayeredPane();

        //set the layout manager
        figure1.setLayoutManager(new XYLayout());

        //init size
        figure1.setSize(300, 300);
        
        //set constraint of DiagramModel
        ((DiagramModel)getModel()).setConstraint(new Rectangle(0,0,300,300));

        return figure1;
    }

    /**
     * Installs EditPolicies to react on User-Interactions
     */
    @Override
    public void createEditPolicies() {

        installEditPolicy(EditPolicy.LAYOUT_ROLE, new MoveableEditPolicy());

    }

    /**
     * Returns the corresponding Models list of Children
     * 
     * @return list List of children
     */
    @Override
    protected List<DiagramElementModel> getModelChildren() {
        return ((DiagramModel) getModel()).getChildren();
    }

    /**
     * Activates this EditPart, used to register Listeners
     */
    @Override
    public void activate() {
        ((DiagramModel) getModel()).addPropertyChangeListener(this);
        super.activate();
    }

    /**
     * Deactivates the EditPart, used to unregister Listeners
     */
    @Override
    public void deactivate() {
        ((DiagramModel) getModel()).removePropertyChangeListener(this);
        super.deactivate();
    }

    /**
     * Reacts on Changes made to the corresponding model
       * 
     * @param event
     *            PropertyChangeEvent fired by an DiagramModel
       *            
     * @see model.DiagramModel.firePropertyChange()
       * 
     * @see java.beans.PropertyChangeListener#propertyChange(java.beans.PropertyChangeEvent)
     */
    public void propertyChange(PropertyChangeEvent evt) {

        if (evt.getPropertyName().equals("SCALE")){ //$NON-NLS-1$

            ((DiagramModel)getModel()).setConstraint((Rectangle)evt.getNewValue());
           }
                
        if (evt.getPropertyName().equals(DiagramModel.DIAGRAM_CONSTRAINT_CHANGED)){

            Rectangle r = ((DiagramElementModel) getModel()).getConstraint();
            getFigure().setBounds(r);
            
//            org.eclipse.swt.graphics.Rectangle r1 = getViewer().getControl().getShell().getBounds();
            
//            InfoscreenModel ism = (InfoscreenModel) getModelChildren().get(0);
            
//            Rectangle r2 = ism.getConstraint();
            
//            ism.setConstraint(new Rectangle(0,r1.x+r1.height-r2.height,r2.width,r2.height));
            
//            org.eclipse.swt.graphics.Rectangle r1 = new org.eclipse.swt.graphics.Rectangle(r.x, r.y, r.width, r.height);
//            getViewer().getControl().getShell().setBounds(r1);
        }
        
        // new element added to the Model
        if (evt.getPropertyName().equals("newElement")) { //$NON-NLS-1$
                       
            calculateSize(((DiagramModel)getModel()).getChildren());
            refreshChildren();
            refreshVisuals();


            // Causes Graph to re-layout
            ((GraphicalEditPart) (getViewer().getContents())).getFigure()
                    .revalidate();
           }

        // Element deleted
        if (evt.getPropertyName().equals(DiagramModel.D_HOST_REMOVED)) {
            removeChild((EditPart) evt.getOldValue());
            refreshChildren();
            refreshVisuals();
            
           
           }

        // Children resized
        if (evt.getPropertyName().equals(HostEditPart.HOST_CONSTRAINT_CHANGED)) {

            calculateSize(((DiagramModel)getModel()).getChildren());
            
//            updateLayout((HostEditPart) evt.getSource(), (Rectangle) evt
//                    .getOldValue(), (Rectangle) evt.getNewValue());
        }

        // Infoscreen Resized
        if (evt.getPropertyName().equals("CHILDISH_CHANGE")) {

            calculateSize(getModelChildren());

//            updateLayout((EditPart) evt.getSource(), (Rectangle) evt
//                    .getOldValue(), (Rectangle) evt.getNewValue());
        }
    }

    /**
     * Calculates the Multiplier for Scaling
     * @param rectangle
     * @param rectangle2
     */
    public ArrayList<Float> getMultiplier(Rectangle rold, Rectangle rnew) {
        
        float  a = rnew.width,
        			b = rold.width, 
        			c = rnew.height,
        			d = rold.height,
                    			
        			mw = (a / b),
        			mh = (c / d);
        
        ArrayList<Float> a1 = new ArrayList<Float>(); 
        
        a1.add(mw);
        a1.add(mh);
        
        return a1;
    }

    /**
     * Calculates the Models Size by adding up the Childrens size
     * 
     * @param modelChildren
     *            List<ObjectModel>
     * @return The new Bounds
     */
    private Rectangle calculateSize(List<DiagramElementModel> modelChildren) {

        if (modelChildren.size() <= 1) return null;
        
        // Init Rects
        Rectangle bounds = new Rectangle(getFigure().getBounds().getLocation(),new Dimension(0,0));

        // add up children Size
        for (DiagramElementModel dem : modelChildren) {
        	
        	if (dem instanceof HostModel){
        		
        		if (((HostModel)dem).getEditPart() == null)return null;
        		
        		HostEditPart hep = (HostEditPart) ((HostModel)dem).getEditPart();

        		IFigure f = hep.getFigure();
        		bounds.union(f.getBounds());
        	}
        	else if  (dem instanceof InfoscreenModel){
        		InfoscreenEditPart iep = (InfoscreenEditPart) (
        				(InfoscreenModel)dem).getEditPart();
        		bounds.union(iep.getFigure().getBounds());
        	}
        }  
        
        // update the Model
        ((DiagramModel) getModel()).setConstraint(bounds.getCopy());
        
        return bounds;
    }

    /**
     * Updates the Pointlist (Currently unused)
     * 
     * @param part
     * @param oldrect
     * @param newrect
     */
    @SuppressWarnings("unused")
	private void updateLayout(@SuppressWarnings("unused")
    EditPart part, @SuppressWarnings("unused")
    Rectangle oldrect, @SuppressWarnings("unused")
    Rectangle newrect) {

        // Update the Pointlist...

        // Update the View
        ((GraphicalEditPart) (getViewer().getContents())).getFigure()
                .revalidate();
    }

    /**
     * calculates the"end-point" of the rectangle at the left down
     * 
     * @param hm HostModel (possibly null)
     * @param temp Location of rectangle
     * @return "end-point" of the rectangle
     */
    
    public Point cornerLocation(HostModel hm, Point temp) {
        if (hm == null) return null; 
        return new Point(temp.x + hm.getSize().width, temp.y
                + hm.getSize().height);
    }
    
    /**********************************************************/
    
    
    /**
     * Seeks on the view for a ponit to draw the new element (Host)
     * @param HostModel hm (Currently unused)
     * @return point to start painting the new element
     */
    public Point getRectangleLikeLocation(HostModel hm) {
   
    	int minDist = 50;  	
    	
    	Point temp = new Point (0,0);
    	
    	int i =getModelChildren().size();
    	
    	if (i==1) return  temp;
    	
    	int z = (int)Math.ceil(Math.sqrt(i));
    	
    	int maxElements = z*z;
    	
    	if (i==maxElements){
    		Rectangle r = getModelChildren().get(i-1).getConstraint();
    		return new Point(r.x+r.width+minDist, r.y);
    		//Rechteck komplett
/*TODO    		Rectangle r = getModelChildren().get(i-1).getConstraint();
    		Rectangle rabove = getModelChildren().get((z-1)*(z-1)).getConstraint();
    
			if (r.y < (rabove.y+rabove.height)){
				
				for (int j=maxElements-z+1;maxElements>j;j++){
					getModelChildren().get(j).setLocation(new Point(r.x,rabove.y+rabove.height+50));
				}
				r.y = rabove.y+rabove.height+minDist;
			}
//			System.out.println("Unten Rechts: "+i+" "+z+" "+maxElements);
			return new Point(r.x+r.width+minDist,r.y);*/
    	}
    	else if (i <= maxElements-z){
    		if (i==((z-1)*(z-1)+1)){
    			//rechts oben
    			Rectangle r = getModelChildren().get(i-1).getConstraint();
    	return new Point(r.x+r.width+minDist,0);
    			//	Rectangle r = getModelChildren().get((z-2)*(z-2)+1).getConstraint();
//    			temp.x = r.x+r.width+minDist;
//    			System.out.println("Rechts Oben: "+i+" "+z+" "+maxElements);
//    			return temp;
    		}
    		else{
    			Rectangle r = getModelChildren().get(i-1).getConstraint();
    			temp.x = r.x;
    			temp.y = r.y+r.height+minDist;
//    			System.out.println("Rechts: "+i+" "+z+" "+maxElements);
    			return temp;
    		//nimm vorgänger und setz drunter
    		}
    	}
    	else if (i > maxElements-z+1){
    		//nimm vorgänger und setz daneben
    		Rectangle r = getModelChildren().get(i-1).getConstraint();
			temp.x = r.x+r.width+minDist;
			temp.y = r.y;
//			System.out.println("Unten: "+i+" "+z+" "+maxElements);
			return temp;
    	}
    	else if (i==maxElements-z+1){
    		//setze unten links unter (z-1)^2-z+1
 		Rectangle r = getModelChildren().get((i-1)).getConstraint();
 		return new Point(0, r.y+r.height+minDist);
    		/*  		Rectangle r = getModelChildren().get((z-2)*(z-2)+(z-1)).getConstraint();
			temp.x = 0;//r.x;
			temp.y = r.y+r.height+minDist;
//			System.out.println("Unten Links: "+i+" "+z+" "+maxElements);
			return temp;*/
    	}
//    	System.out.println("None: "+i+" "+z+" "+maxElements);
    	return null;
    }
    
 public Point getAutomaticLocation(HostModel hm) {
	/*TODO zwei verschiedene DoppelSchleifen, eine fr x und eine fr y.
	 * fr bessere ausbeute an Screen-Platz.
	 */
	 int minDist = 25;
	 if (minDist == 25) return getRectangleLikeLocation(hm);
	 //default Hostsize
	 Dimension defaultHost = new Dimension(30,5);
 	Point temp = new Point(0,0);
 	Dimension limit = getLimit();
 	Rectangle leftes = null;//new Rectangle(0,0,0,0);
     for (int j = temp.y; j < limit.height+minDist; j++){
//    	 if (leftes != null) {j= leftes.y+leftes.height+minDist;
//    	 leftes = null;}
    	 for (int i = temp.x; i< limit.width-minDist; i++) {
    		 Point dinb = new Point(i,j);
    		if (getLimit().width > getLimit().height) {limit.height+=minDist;}
    		 DiagramElementModel dem = findModelAt(i,j);
    	 
    	 if (findModelAt(i,j)!=null) {
    		 if (leftes == null) leftes = dem.getConstraint();
    		 i= dem.getConstraint().x+dem.getConstraint().width+minDist;}   	 
    	 
    	 else if (findModelAt(i,j) == null) {
    		 if (hm == null) {
    			  if ( checkCollision(new Rectangle(dinb,defaultHost))== null) { return new Point(i,j);}    		 
    		 else if ( checkCollision(new Rectangle(dinb,defaultHost))!= null) { return insert(hm,new Rectangle (dinb,defaultHost));}}

   		 else if (hm!= null){
    			 Rectangle hmIntend = hm.getConstraint();
    			 
    			 hmIntend.setLocation(i,j);
    		 
    			 if (checkCollision(hmIntend) == null) return new Point (i,j);
    			 else if (checkCollision(hmIntend) != null) 
    				 return insert(hm, hmIntend);
    		 	}
    		 }
    	 }
     }
  	return null;//new Point(getLimit().width+minDist, temp.y);
 }
	
     /**
    	    	 
     	limit = getLimit();         	
     	//TODO DEBUG! last setted
      if ( temp.y >= limit.height) {return new Point(limit.width+minDist,temp.y);}    	
     	 else if (temp.x >= limit.width ) { //||( limit.x>limit.y && !(temp.y >= limit.y))
		      System.out.println(temp + " + " + limit);
     			temp.x = 0;
     			temp.y += minDist;//leftes.getLocation().y+leftes.getSize().height+minDist;
     			if (left) temp.y += /*leftes.getLocation().y+leftes.getSize().height+minDist;
     			left = false;		
     	 }
	
     	else if (findModelAt(temp) != null) {

     		if (!left) {leftes.setBounds(findModelAt(temp).getConstraint()); left = true;}
     		temp.x += /*findModelAt(temp).getLocation().x+ findModelAt(temp).getSize().width+minDist;
     		
     	/*	temp.y +=lop;
     			lop+=1;
              	System.out.println(temp);
     	}
     	else if (findModelAt(temp) == null) {
    			if (hm==null) return temp;
    			else {
    				Rectangle save = hm.getConstraint();
    				hm.setLocation(temp);
    				if (checkCollision(hm) == null) {return temp;}
    						if (checkCollision(hm) != null)	{
    							Rectangle rec = checkCollision(hm).getConstraint();
    							temp.x+=minDist;
    							//temp.x =rec.x+rec.width+minDist;	
    							hm.setConstraint(save);  						
    						}
    				}
    		}
     		 //TODO: Nur ne Stellschraube...
     temp.x+=1;
   // temp.y+=1;
     	 */


/**
 * starts the method DiagramElementModel findModelAt(Point temp) with the overgiven values
 * 
 * @param x position of point on x - axle
 * @param y position of point on y - axle
 * @return findModelAt
 */  
    public DiagramElementModel findModelAt(int x, int y) {
        return findModelAt(new Point(x, y));
    }

    /**
     * searches on the view for an other HostModel at the point temp
     * 
     * @param temp the point that is possiply occured
     * @return the occupying model or null, when point is free
     */
    public DiagramElementModel findModelAt(Point temp) {
        for (int i = 0; i <= ((DiagramModel) getModel()).getChildren().size() - 1; i++) {
            if (temp.x >= ((DiagramModel) getModel()).getChildren().get(i)
                    .getLocation().x
                    && temp.x <= ((DiagramModel) getModel()).getChildren().get(
                            i).getSize().width
                            + ((DiagramModel) getModel()).getChildren().get(i)
                                    .getLocation().x
                    && temp.y >= ((DiagramModel) getModel()).getChildren().get(
                            i).getLocation().y
                    && temp.y <= ((DiagramModel) getModel()).getChildren().get(
                            i).getSize().height
                            + ((DiagramModel) getModel()).getChildren().get(i)
                                    .getLocation().y)
                return ((DiagramModel) getModel()).getChildren().get(i);
        }
        return null;
    }
           
    /**
     * Checks if there is collision between DiagramModel und HostModel
     * @param dem DiagramModel
     * @param hm HostModel
     * @return true if there is collision, otherweise false
     */
    public boolean checkCollision(Rectangle dem, Rectangle hm) {
    	if (dem!=hm)
   /* 	if (findModelAt(dem.getLocation()) == hm 
    	|| findModelAt(dem.getLocation().x,dem.getLocation().y+dem.getSize().height) == hm
    	|| findModelAt(dem.getLocation().x+dem.getSize().width, dem.getLocation().y) == hm
        || findModelAt(dem.getLocation().x+ dem.getSize().width,dem.getLocation().y+ dem.getSize().height) == hm)
    		 return true;*/
    	if(!dem.isEmpty() && !hm.isEmpty())
    		if (dem.getLocation().y+ dem.getSize().height>= hm.getLocation().y) 	
    	   if ( dem.getLocation().y <=  hm.getLocation().y + hm.getSize().height  )         
        if    ( dem.getLocation().x+ dem.getSize().width>= hm.getLocation().x)
        if    ( dem.getLocation().x <= hm.getLocation().x + hm.getSize().width)
    		return true;       
	return false;
}
    
    /**
     * Returns true if any Host collides with Infoscreen, else when not.
     * 
     * @param model
     * @return blokading Host
     */
    public boolean infoscreenBlocked(HostModel model) {
	  return checkCollision(((DiagramModel) getModel()).getChildren().get(0), model);
}
    /**
     * Returns true if any Host collides with Infoscreen, else when not.
     * 
     * @param model
     * @return blokading Host
     */
    public boolean infoscreenBlocked(Rectangle rec) {
	  return checkCollision(((DiagramModel) getModel()).getChildren().get(0).getConstraint(), rec);
}
   
        /**
         * positionates the Hosts in a specially order, given by an 
         * attribute.     
         *      
         * @param modus a string thad defines the way to sort
         */
        public void autoHierarchy(String modus) {
        	int minDist = 25;
        	if (modus == null) modus = "width";
        	int biggest[] = getHierarchy(modus);
        	Point p = new Point (0,0);
        	  for (int i: biggest){
        		  p = getAutomaticLocation(null);
        		  if (((DiagramModel)  getModel()).getChildren().get(i) instanceof InfoscreenModel) {;}
        		  else ((HostModel) ((DiagramModel)  getModel()).getChildren().get(i)).setLocation(p);   		  
        	   }       
       	  for (int i: biggest){
      		  	for (int j: biggest) {       	
     	  			if ((j>1 && j>1) && checkCollision( ((DiagramModel) getModel()).getChildren().get(i).getConstraint(),        		  					
    ((DiagramModel) getModel()).getChildren().get(i-1).getConstraint())) {
        	Rectangle tr =   ((DiagramModel)   getModel()).getChildren().get(i-1).getConstraint();       		  				
    tr.x+=minDist+tr.width;((DiagramModel)  getModel()).getChildren().get(i).setLocation(tr.getLocation());}
            	
        		/*	if (!(((DiagramModel) 

    getModel()).getChildren().get(j) instanceof InfoscreenModel))	if ( checkCollision( ((DiagramModel)   getModel()).getChildren().get(i).getConstraint(),  ((DiagramModel) getModel()).getChildren().get(j).getConstraint())) {
        			Rectangle tr = ((DiagramModel)   getModel()).getChildren().get(i).getConstraint();
        			tr.x+=minDist+tr.width;
        			((DiagramModel)  getModel()).getChildren().get(j).setLocation(tr.getLocation());}}*/
        		  	}
        	  }
        }
               
        /**
         * arranges the Hosts, Components and Objects close to the structure of an rectangle
         *
         */
        public void autoSqrt(){
        	for (int i =1; i< ((DiagramModel)  getModel()).getChildren().size(); i++){
        		((DiagramModel)  getModel()).getChildren().get(i).setLocation(
        				getRectangleLikeLocation((HostModel)((DiagramModel)  getModel()).getChildren().get(i)) );
        	//TODO cross[]-Methoden fr Infoscreen einfgen
        		/*while (save) {
        			if (checkCollision(((DiagramModel)  getModel()).getChildren().get(i))!=null) {
        		((DiagramModel)  getModel()).getChildren().get(i).getLocation().x+=minDist;
        		}
        			else save = true;
        		}*/
        	}       	
        
        }
        
        public void autoSqrt2(){
        	int minDist = 30;
        	Rectangle dinb = new Rectangle(0,0,0,0);
        	int DiagramIndex = ((DiagramModel)  getModel()).getChildren().size();
        	int sqrt= (int) Math.abs(Math.sqrt(DiagramIndex));
        	((DiagramModel)  getModel()).getChildren().get(1).setLocation(new Point(0,0));
    		for (int i=2; i< DiagramIndex; i++){
    			Rectangle temp = ((DiagramModel)  getModel()).getChildren().get(i-1).getConstraint();
 			
    			if   (i%sqrt==1) { temp.setBounds( ((DiagramModel)  getModel()).getChildren().get(i-1).getConstraint());
    			((DiagramModel)   getModel()).getChildren().get(i).setLocation(new  Point(0,temp.y+temp.height+minDist));}	
    			
    			else { 
    				dinb.setBounds(((DiagramModel) getModel()).getChildren().get(i-1).getConstraint());
    				dinb.x =(temp.x+temp.width+minDist);}
        				if (checkCollision(((DiagramModel) getModel()).getChildren().get(0).getConstraint(),dinb)) {
        					dinb.x = ((DiagramModel) getModel()).getChildren().get(i).getLocation().x+
        					((DiagramModel)  getModel()).getChildren().get(i).getSize().width+minDist;}
        				
    		else ((DiagramModel)  getModel()).getChildren().get(i).getLocation().x  =(temp.x+temp.width+minDist);	
    		}   
        }
        
        public Point insert2 (HostModel hm){
        	int minDist = 25;
        	Point temp = new Point (0,0);
        	Rectangle limit = new Rectangle(temp, getLimit());
        	for (int j = limit.y; j<= limit.y+limit.height; j++) {
        		for (int i=limit.x; i<=limit.x+limit.width;i++){
        			Rectangle found = findModelAt(i,j).getConstraint();
        //			if (left == null && found!= null){left.setBounds(found.getCopy());
        			
        			if ( found != null) {i=found.x+found.width+minDist;}
        			else {;
        			//	if ()
        			}
       		}   		
        	} 	
        	return null;
        }

        /**
         * when its definitve, that HostModel hm blokades the Infoscreen 
         * or any other Host, one of the concerned shall be  repositionated.
         * 
         * @param hm
         * @param hmPosition constraints of hm. in the first they are the same 
         * but in often iterations of insert() it may change
         */
        public Point insert(HostModel hm, Rectangle hmPosition){
        	int minDist = 25;
        	Rectangle blockade = checkCollision(hmPosition).getConstraint();
        	
        	//when infoscreen is blocked  
        	 if (hmPosition.intersects(((DiagramModel)  getModel()).getChildren().get(0).getConstraint()))
        	 { 	Rectangle infoScreen = ((DiagramModel)  getModel()).getChildren().get(0).getConstraint();

        	 //repositionate        		
        		 if (hmPosition.x> (int) ((infoScreen.x+ infoScreen.width)/2)) {
        			hmPosition.x= infoScreen.x+ infoScreen.width + minDist;
        		}
        		else {hmPosition.x+=minDist;}
        		
        			if (hmPosition.y> (int) ((infoScreen.y+ infoScreen.height)/2)) 
        			{
        			hmPosition.y= infoScreen.y+ infoScreen.height + minDist;
        		}
        			else {hmPosition.y+=minDist;}
        		 
        		 //check hm again at its new position
        		 if (checkCollision(hmPosition) != null) {hmPosition.setLocation(insert(hm,hmPosition));}
        		 return hmPosition.getLocation();
    			 }
        	 
        		
           	 //when hm is at left
        		if (crossLeft(blockade)) {
        			hmPosition.x= blockade.x   + blockade.width+ minDist;

            		//check again - the new position
            		if (checkCollision(hmPosition) == null)  insert(hm,hmPosition);    	
        		}
        		
           	 //when hm is at top    	
        		if (crossTop(blockade)) {
        			hmPosition.y= blockade.y +   blockade.height+ minDist;
            		//check again - the new position
            		if (checkCollision(hmPosition) == null)  insert(hm,hmPosition);    	
        		}
        		else {
        			//when hm intersects on other side, the blocked Element wil be pushed
        		 if (crossRight(blockade)) {
        			 hmPosition.x = blockade.x +   blockade.width + minDist; }
        		 if (crossBottom(blockade)) {
        			 hmPosition.y = blockade.y +  blockade.height + minDist;}
        		 
        	/*	 int list[] = movingList(hmPosition);
    			 for (int i=1; i < ((DiagramModel)  getModel()).getChildren().size();i++) {
    				 if (list[i] != 999) {
    					 ((DiagramModel)  getModel()).getChildren().get(i).getLocation().x+=hmPosition.width;
    					 ((DiagramModel)  getModel()).getChildren().get(i).getLocation().y+=hmPosition.height;
    						}
    				 }
    		 return hmPosition.getLocation();*/
        		}
        		if (((DiagramModel)  getModel()).getChildren().size()>2) 
        			for (int i=1; i< ((DiagramModel)  getModel()).getChildren().size();i++) {
        			if (checkCollision(((DiagramModel)  getModel()).getChildren().get(i)) == null)  insert(
        					(HostModel) ((DiagramModel)  getModel()).getChildren().get(i),
        					((DiagramModel)  getModel()).getChildren().get(i).getConstraint());    	            		
        		}
        		return hmPosition.getLocation();
        }
        /**
         * returns true, when this Host enters the Rectangle blockade with its bottom side. 
         * 
         * @param blockade
         * @return
         */
        private boolean crossBottom(Rectangle blockade) {
    		Rectangle thisDem = ((DiagramElementModel) getModel()).getConstraint(); 
    		if ( blockade.y < thisDem.y
    				&& thisDem.y < blockade.y+blockade.height) {
    			return true;}
    		return false;
    	}

        /**
         * returns true, when this Host enters the Rectangle blockade with its left side. 
         * 
         * @param blockade
         * @return
         */
    	private boolean crossLeft(Rectangle blockade) {
    		Rectangle thisDem = ((DiagramElementModel) getModel()).getConstraint(); 
    		if ( blockade.x < thisDem.x
    				&& thisDem.x < blockade.x+blockade.width) {
    			return true;}
   		return false;
    	}

        /**
         * returns true, when this Host enters the Rectangle blockade with its right side. 
         * 
         * @param blockade
         * @return
         */
    	private boolean crossRight(Rectangle blockade) {
    		Rectangle thisDem = ((DiagramElementModel) getModel()).getConstraint(); 
    		if ( blockade.x < thisDem.x +thisDem.width
    				&& thisDem.x +thisDem.width < blockade.x+blockade.width) {
    			return true;}
    		return false;
    	}

        /**
         * returns true, when this Host enters the Rectangle blockade with its top side. 
         * 
         * @param blockade
         * @return
         */
    	private boolean crossTop(Rectangle blockade) {
    		Rectangle thisDem = ((DiagramElementModel) getModel()).getConstraint(); 
    		if ( blockade.y < thisDem.y +thisDem.height
    				&& thisDem.y +thisDem.height < blockade.y+blockade.height) {
    			return true;}    		
    		return false;
    	}
 
    	/**
    	 * gets the constraints of the first element in the list that intersects with the 
    	 * DiagramElementModel
    	 * 
    	 * @param hmPosition
    	 * @return
    	 */
    	public Rectangle intersectsInList(Rectangle hmPosition){
        	for (DiagramElementModel dem: ((DiagramModel)   getModel()).getChildren()){
        		if (dem.getConstraint().intersects(hmPosition)) {
        			return dem.getConstraint();	
        		}   		
        	}
        return null;
        }
       
    	/**
    	 * Returns a list of the order the elements shall be sorted
    	 * 
    	 * TODO: �dern zu int[]
    	 * 
    	 * @param start
    	 * @return
    	 */
    	public int[] movingList(Rectangle start){
    		int llist[] = new int[((DiagramModel)  getModel()).getChildren().size()];
    		Rectangle movArea = new Rectangle(start.x,  start.y, getLimit().width, getLimit().height);
    		for ( int i = 1; i<((DiagramModel)  getModel()).getChildren().size();i++){
    			if (((DiagramModel)   getModel()).getChildren().get(i).getConstraint().intersects(movArea)) {
//    				llist[i] = ((DiagramModel)  getModel()).getChildren().get(i).getConstraint();
    				llist[i] = i;}
    				else {llist[i] =999;}
    			
    		}		    		
    		return llist;
    	}
    	
        /**
         * searches in the ModelList for the constraints of a Model that intersects with  hm
         * 
         * @param HostModel hm
         * @return HostModel
         */
        public DiagramElementModel checkCollision(Rectangle rec) {    	
        	if (infoscreenBlocked(rec)) return ((DiagramModel)  getModel()).getChildren().get(0);   	
        	for ( DiagramElementModel nu: ((DiagramModel)  getModel()).getChildren()) {
                if ((nu != ((DiagramModel)    getModel()).getChildren().get(0))
        		&& checkCollision(nu.getConstraint(), rec) ==  false)
                    return  nu;            
            } 
            return null;
        }
        
        /**
         * searches in the ModelList for a Model that intersects with  hm
         * 
         * @param HostModel hm
         * @return HostModel
         */
        public DiagramElementModel checkCollision(HostModel hm) {    	
        	if (infoscreenBlocked(hm)) return ((DiagramModel)   getModel()).getChildren().get(0);   	
        	for ( DiagramElementModel nu: ((DiagramModel)  getModel()).getChildren()) {
                if ((nu != ((DiagramModel)   getModel()).getChildren().get(0) && nu !=hm)
        		&& checkCollision((HostModel)nu, hm) == false)
                    return  nu;            
            } 
            return null;
        }

        public DiagramElementModel checkCollision(DiagramElementModel dem) {   
        	return checkCollision((HostModel) (dem));
        }
        
        /**
         * returns true if the DiagramElementModels intersect
         * 
         * @param DiagramElementModel dem
         * @param DiagramElementModel hm
         * @return boolean
         */
        public boolean checkCollision(DiagramElementModel dem, DiagramElementModel dem1) {
    	if(dem.getLocation() != null && !dem.getSize().isEmpty()  && dem1.getLocation() != null && !dem1.getSize().isEmpty())
        	if (dem.getLocation().y+ dem.getSize().height>=   dem1.getLocation().y
                && dem.getLocation().y <=  dem1.getLocation().y +   dem1.getSize().height           
                && dem.getLocation().x+ dem.getSize().width>=   dem1.getLocation().x
                && dem.getLocation().x <= dem1.getLocation().x +  dem1.getSize().width)
                return true;       
    	return false;
    }
            
        /**
         * Returns true if the HostModels intersect.
         * 
         * @param HostModel dem
         * @param HostModel hm
         * @return boolean
         */
        public boolean checkCollision(HostModel dem, HostModel hm) {
        	if (dem!=hm)
       /* 	if (findModelAt(dem.getLocation()) == hm 	
        ||  findModelAt(dem.getLocation().x,dem.getLocation().y+dem.getSize().height) == hm 
        || findModelAt(dem.getLocation().x+dem.getSize().width,  dem.getLocation().y) == hm 
        || findModelAt(dem.getLocation().x+  dem.getSize().width,dem.getLocation().y+ dem.getSize().height) ==  hm) 
        return true;*/
        	if(!dem.getConstraint().isEmpty() &&  !hm.getConstraint().isEmpty())
        		if (dem.getLocation().y+ dem.getSize().height>=   hm.getLocation().y) 	
        	   if ( dem.getLocation().y <=  hm.getLocation().y +  hm.getSize().height  )         
            if    ( dem.getLocation().x+ dem.getSize().width>=   hm.getLocation().x)
            if    ( dem.getLocation().x <= hm.getLocation().x +   hm.getSize().width)
        		return true;       
    	return false;
    }
               
        /**
         * returns a list of a hierarchy of the Hosts. Sorted by a  choosable attribute.
         * For now there are two ways possible: the widest first or  the highest first.
         * Other modi like the number of connections can be implementet later
         * 
         * @param thing
         * @return list of sorted indices
         */
        public int[] getHierarchy(String thing){
        	
        	if (thing == "width") return getWidthHierarchy();
      	
        	else if (thing == "height")return getWidthHierarchy();

        	else if (thing == "size")  return getSizeHierarchy();       		
        	
        	else if (thing == "connections") return getConnectionsHierarchyInt();
        
        	else return null;
        
         }
        
        public int[] getWidthHierarchy(){
	int size =((DiagramModel) getModel()).getChildren().size();
	int list[] = new int[size];
	int temp=0;
	for(int i = 0; i< size; i++) {
		list[i] = i;
	}
  	for (int i = 1; i< size; i++) {    		  
   	 for (int j = 1; j< size-2; j++){	//TODO Achtung
             if ( ((DiagramModel) getModel()).getChildren().get(j).getSize().width<
   		((DiagramModel)  getModel()).getChildren().get(j+1).getSize().width)
           	  temp = list[j]; list[j] =list[j+1];list[j+1] =temp;}  }
  	  	
return list;
	}
	
        public int[] getSizeHierarchy(){
        	int size =((DiagramModel) getModel()).getChildren().size();
        	int list[] = new int[size-1];
        	for (int i=2; i< size;i++) {
    			DiagramElementModel dem = ((DiagramModel) getModel()).getChildren().get(i);
    			int countBig =1;
    			if (dem.getConstraint().width*dem.getConstraint().height > 
    			((DiagramModel) getModel()).getChildren().get(countBig).getConstraint().width*        				
    			((DiagramModel) getModel()).getChildren().get(countBig).getConstraint().height)
    		countBig=i;
        	}
        	return list;  	
        }
        
        public int[] getHeightHierarchy(){
        	int size =((DiagramModel) getModel()).getChildren().size();
        	int list[] = new int[size-1];
    	  	for (int i = 1; i< size; i++) {    		  
       		 for (int j = 1; j< size-1; j++){
       	          if ( ((DiagramModel) getModel()).getChildren().get(j).getSize().height<
       			((DiagramModel)   getModel()).getChildren().get(j+1).getSize().height)
       	        	  list[j] =j+1;list[j+1] =j;}  }  
    	return list;  	
        }
        
        public boolean getQ() {
        	@SuppressWarnings("unused") Point temp = new Point(); 
//        	DiagramElementModel ex =new DiagramElementModel();
        	
        	int xx=0; int yy=0;
        	//allelements, h�chste x
        	for (DiagramElementModel dem: ((DiagramModel)   getModel()).getChildren() ) {
        		xx =  Math.max((dem.getLocation().x+dem.getSize().width),xx);
        		yy =  Math.max((dem.getLocation().y+dem.getSize().height),yy);
        			}
        	if (xx > yy) return true;
        	
        	//entscheiden, ob "enter"
        	return false;
        }

            /**
             * creates a point with the highest x- and y- values, got by the values of 
             * all Hosts in the Diagram
             * 
             * @return point
             */
            public Dimension getLimit(){
            	Rectangle right = new Rectangle(0,0,0,0);	
            	Rectangle down = right.getCopy();
            	for (DiagramElementModel dem: ((DiagramModel)   getModel()).getChildren() ) {
            		if (dem.getLocation().x+dem.getSize().width >   right.getLocation().x+right.getSize().width)   
            			right.setBounds(dem.getConstraint());
            		if (dem.getLocation().y+dem.getSize().height >  down.getLocation().y+down.getSize().height)   
            			down.setBounds(dem.getConstraint());
            	}
        		
            	return new Dimension (   right.getLocation().x+right.getSize().width+10, down.getLocation().y+right.getSize().height+10  );
        }
            /**
             * TODO: Verifizieren, da�die LocationSetzung nicht zu Problemen fhrt
         * @return
         */
            private Rectangle workplane() {
        	int minDist = 25;
            Rectangle workPlane = new Rectangle (0,0,0,0);
            for (DiagramElementModel dem: ((DiagramModel) getModel()).getChildren()){
                workPlane.width+=dem.getConstraint().width+minDist;
                workPlane.height+=dem.getConstraint().height+minDist;
                workPlane.x+=dem.getConstraint().width+minDist;
                workPlane.y+=dem.getConstraint().height+minDist;
            }
            
            // TODO Auto-generated method stub
            return workPlane;
        }

            /**
             * 
             * 
             * @return
             */
            public int[] getConnectionsHierarchyInt() {
            int list[] = new int[((DiagramModel) getModel()).getChildren().size()];
            int connectionsCount=0;
            
            // get HostModel with most Connections
            for (int j=0; j< list.length; j++) {
                connectionsCount=0;
            for (int i=1; i< ((DiagramModel) getModel()).getChildren().size(); i++){
                boolean isInList = false;

                for (int g=0; g< list.length; g++){
                //how to break loop?
                    if (g==i) isInList = true;    
                }
                HostModel hm = (HostModel) (((DiagramModel) getModel()).getChildren().get(i));
                HostModel con = (HostModel) (((DiagramModel) getModel()).getChildren().get(i));
                
                if (!isInList && hm.getEndConnections().size() + hm.getStartConnections().size() >
                con.getEndConnections().size() + con.getStartConnections().size())
                    connectionsCount=i;
                }
            list[j] = connectionsCount;//((DiagramModel) getModel()).getChildren().get(connectionsCount).getConstraint();
            }
              //  sourceConnections
            
           // ((HostModel) getModel()).getChildren().get(3).getEndConnections().size()+  ;
                return list;
            /* 
             * get mostConnections,
             * New List: get Targets
             * sort Targets nach Anzahl Connections (get Hierarchy)
             * wenn obere Rekursion fertig, weiter:
             * if ( get(i) not in List &&)
             */        
        }

            public Rectangle[] getConnectionsHierarchy() {
                Rectangle list[] = new Rectangle[((DiagramModel) getModel()).getChildren().size() ];
                int connectionsCount=0;
                //TODO auch Connections von&zu Kindern z�len.. for all children..
                // get HostModel with most Connections
                for (int j=0; j< list.length; j++) {
                for (int i=1; i< ((DiagramModel) getModel()).getChildren().size(); i++){
                    boolean isInList = false;
                    for (int g=0; g< list.length; g++){
                    //how to break loop?
                        if (g==i) isInList = true;    
                    }                       
                    if (!isInList && ((HostModel) (((DiagramModel) getModel()).getChildren().get(i))).getEndConnections().size() + 
                            ((HostModel) (((DiagramModel) getModel()).getChildren().get(i))).getStartConnections().size() >
                    ((HostModel) (((DiagramModel) getModel()).getChildren().get(connectionsCount))).getEndConnections().size() + 
                    ((HostModel) (((DiagramModel) getModel()).getChildren().get(connectionsCount))).getStartConnections().size())
                        connectionsCount=i;}
                list[j] = ((DiagramModel) getModel()).getChildren().get(connectionsCount).getConstraint();
                }
                  //  sourceConnections
                
               // ((HostModel) getModel()).getChildren().get(3).getEndConnections().size()+  ;
                    return list;
                /* 
                 * get mostConnections,
                 * New List: get Targets
                 * sort Targets nach Anzahl Connections (get Hierarchy)
                 * wenn obere Rekursion fertig, weiter:
                 * if ( get(i) not in List &&)
                 */        
            }
            /**
             * TODO: erweitern um KristallStruktur, durch besseres Array:
             * targetConnections in 2D-Array und merken, wieviele richtungen schon "besetzt sind
             * TODO: caseliste ersetzen durch Modulo: dann werden zwar die sp�eren an die selben 
             * stellen gesetzt, wie die ersten, aber k�nen dann ja per "insert verschoben werden
             * TODO: ersetze setLocation() durch "setCenter"
             */
            public void autoStarLayout() {
		int minDist = 25;

		Rectangle workplane = workplane();
		DiagramElementModel Infoscreen = ((DiagramModel) getModel()).getChildren().get(0);
		// TODO: ist Umsetzen vom Infoscreen wirklich n�ig?
//		Infoscreen.setLocation(new Point(0,0));

		/*	Infoscreen.getLocation().x += workplane.x;
		Infoscreen.getLocation().y += workplane.y;
*/
		int list[] = getHierarchy("width");
		// ((DiagramModel)
		// getModel()).getChildren().get(list[0]).setLocation(workplane.getCenter());

//		for (int i = 0; i < list.length; i++) {
	for (int i: list){
		//
			System.out.println(i + " QQQQQQQQQQQQQQQ  "+list.length);
			if (list[i]!=0){
				DiagramElementModel now = ((DiagramModel) getModel()).getChildren().get(list[i]);
				DiagramElementModel last = ((DiagramModel) getModel()).getChildren().get(list[i - 1]);
				DiagramElementModel last2 = ((DiagramModel) getModel()).getChildren().get(list[i - 2]);
				DiagramElementModel last1 = ((DiagramModel) getModel()).getChildren().get(list[i - 1]);
				DiagramElementModel last3 = ((DiagramModel) getModel()).getChildren().get(list[i - 3]);
				DiagramElementModel last4 = ((DiagramModel) getModel()).getChildren().get(list[i - 4]);
				
				switch (i % 9) {			
			// the rightwinkles
			case 0:
				((DiagramModel) getModel()).getChildren().get(list[i]).setLocation(workplane.getCenter());
			case 1: {
				Point t = last.getLocation();
				t.x = last.getLocation().x + last.getSize().width + minDist;
				now.setLocation(t);
				if (checkCollision(now.getConstraint()) != null) {
					insert((HostModel) now, ((HostModel) now).getConstraint());}
			}
			case 2: {
				Point t = last2.getLocation();
				t.x = last2.getLocation().y + last1.getSize().height + minDist;
				now.setLocation(t);
				if (checkCollision(now.getConstraint()) != null) {
					insert((HostModel) (now), ((HostModel) (now)).getConstraint());}
			}
			case 3: {
				Point t = new Point(last3.getLocation());
				t.x = last3.getLocation().x - now.getSize().width - minDist;
				now.setLocation(t);
				if (checkCollision(now.getConstraint()) != null) {
					insert((HostModel) now, ((HostModel) now).getConstraint());}
			}
			case 4: {
				Point t = new Point(last4.getLocation());
				t.x = last4.getLocation().y - now.getSize().height - minDist;
				now.setLocation(t);
				if (checkCollision(now.getConstraint()) != null) {
					insert((HostModel) (now), ((HostModel) (now)).getConstraint());}
			}
			// the diagonales
			case 5: {
				Point t = new Point(last4.getLocation());
				t.x = last4.getLocation().y + last4.getSize().height + minDist;
				now.setLocation(t);
				if (checkCollision(now.getConstraint()) != null) {
					insert((HostModel) (now), ((HostModel) (now)).getConstraint());}
			}
			case 6: {
				Point t = last4.getLocation();
				t.x = last4.getLocation().x - now.getSize().width - minDist;
				now.setLocation(t);
				if (checkCollision(now.getConstraint()) != null) {
					insert((HostModel) (now), ((HostModel) (now)).getConstraint());}
			}
			case 7: {
				Point t = last4.getLocation();
				t.x = last4.getLocation().y - now.getSize().height - minDist;
				now.setLocation(t);
				if (checkCollision(now.getConstraint()) != null) {
					insert((HostModel) (now), ((HostModel) (now)).getConstraint());}
			}
			case 8: {
				Point t = last4.getLocation();
				t.x = last4.getLocation().x + last4.getSize().width + minDist;
				now.setLocation(t);
				if (checkCollision(now.getConstraint()) != null) {
					insert((HostModel) (now), ((HostModel) (now)).getConstraint());}
			}
				}
			}
                    /*
					 * get Center get "centers" Position x+ get centers
					 * posititnp y+ get centers posititnp x- get centers
					 * posititnp y- get i-2 (centers left) Position, y+ get
					 * (i-2) (centers bottom) Position x- get (i-2) (centers
					 * left) Position y- get (i-2) (centers top) Position x+ vor
					 * den Eingen Abstand/Collision testen
					 */ 
                }
        Dimension resize = getMinimalSize();
		for (DiagramElementModel dem: ((DiagramModel) getModel()).getChildren()) {
			if (!(dem instanceof InfoscreenModel)) {
            dem.getLocation().x-=resize.width;
            dem.getLocation().y-=resize.height;}
        }
for (DiagramElementModel dem: ((DiagramModel) getModel()).getChildren()) {
	if (!(dem instanceof InfoscreenModel) && checkCollision(dem, Infoscreen)) {
		getAutomaticLocation((HostModel) dem);
	}
	
}       
            
            }
			private Dimension getMinimalSize() {
	            	Rectangle right = new Rectangle(new Point (0,0),getLimit());	
	            	Rectangle down = right.getCopy();
	            	for (DiagramElementModel dem: ((DiagramModel)   getModel()).getChildren() ) {
	            		if (dem.getLocation().x <   right.getLocation().x)   
	            			right.setBounds(dem.getConstraint());
	            		if (dem.getLocation().y<  down.getLocation().y)   
	            			down.setBounds(dem.getConstraint());
	            	}
	        		
	            	return new Dimension ( down.width,down.height);
			}

	public void autoStandard() {

		for (DiagramElementModel dem : ((DiagramModel) getModel()).getChildren()) {
			if (!(dem instanceof InfoscreenModel))
				dem.setLocation(new Point(9999, 9999));
		}
		for (int i = 1; i < ((DiagramModel) getModel()).getChildren().size(); i++) {
			Point p = getAutomaticLocation((HostModel) ((DiagramModel) getModel()).getChildren().get(i));
			((DiagramModel) getModel()).getChildren().get(i).setLocation(p);
		}
	}

	public Point insertXX(HostModel hm, Rectangle hmPosition){
    	int minDist = 25;
    	Rectangle blockade = checkCollision(hmPosition).getConstraint();
    	
    	// when infoscreesn is blocked
    	 if (hmPosition.intersects(((DiagramModel)  getModel()).getChildren().get(0).getConstraint()))
    	 { 	Rectangle infoScreen = ((DiagramModel)  getModel()).getChildren().get(0).getConstraint();

    	 // repositionate
    		 if (hmPosition.x> (int) ((infoScreen.x+ infoScreen.width)/2)) {
    			hmPosition.x= infoScreen.x+ infoScreen.width + minDist;
    		}
    		else {hmPosition.x+=minDist;}
    		
    			if (hmPosition.y> (int) ((infoScreen.y+ infoScreen.height)/2)) 
    			{
    			hmPosition.y= infoScreen.y+ infoScreen.height + minDist;
    		}
    			else {hmPosition.y+=minDist;}
    		 
    		 //check hm again at its new position
    		 if (checkCollision(hmPosition) != null) {hmPosition.setLocation(insert(hm,hmPosition));}
    		 return hmPosition.getLocation();
			 }
    	 
    		
       	 //when hm is at left
    		if (crossLeft(blockade)) {
    			hmPosition.x= blockade.x   + blockade.width+ minDist;

        		//check again - the new position
        		if (checkCollision(hmPosition) == null)  insert(hm,hmPosition);    	
    		}
    		
       	 //when hm is at top    	
    		if (crossTop(blockade)) {
    			hmPosition.y= blockade.y +   blockade.height+ minDist;
        		//check again - the new position
        		if (checkCollision(hmPosition) == null)  insert(hm,hmPosition);    	
    		}
    	
    		
    		else {
    			//when hm intersects on other side, the blocked Element wil be pushed
    		 if (crossRight(blockade)) {
    			 hmPosition.x = blockade.x +   blockade.width + minDist; }
    		 if (crossBottom(blockade)) {
    			 hmPosition.y = blockade.y +  blockade.height + minDist;}
	 
    		 int list[] = movingList(hmPosition);
			 for (int i=1; i < ((DiagramModel)  getModel()).getChildren().size();i++) {
				 if (list[i] != 9999999) {
					 ((DiagramModel)  getModel()).getChildren().get(i).getLocation().x+=hmPosition.width;
					 ((DiagramModel)  getModel()).getChildren().get(i).getLocation().y+=hmPosition.height;
						}
				 }
		 return hmPosition.getLocation();
    		}
    		for (int i=1; i< ((DiagramModel)  getModel()).getChildren().size();i++) {
    			if (checkCollision(hmPosition) == null)  insert(hm,hmPosition);    	            		
    		}
    		
    	return null;
    }
}