/*
 * Copyright (C) 2006 Johannes Baumgarten, Bjoern Gebhardt, Mathias Kellner, Stephan Kluge, Svetlana Li, Jurij Merker, Daniel Meyer, Andre Schucht, Ling Wu
 *
 * This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 2 of the License. 
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details. 
 * You should have received a copy of the GNU General Public License along with this program; if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA
 */

package Debug.Visualizer.editparts;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.XYLayout;
import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.draw2d.geometry.Rectangle;

import org.eclipse.gef.EditPolicy;
import org.eclipse.gef.GraphicalEditPart;
import org.eclipse.gef.editparts.AbstractGraphicalEditPart;

import Debug.Visualizer.model.*;
import Debug.Visualizer.model.commands.MoveCommand;
import Debug.Visualizer.editpolicies.MoveableComponentEditPolicy;
import Debug.Visualizer.figure.HostFigure;
import Debug.Visualizer.gui.VisualizerColors;

/**
 * Controls the behavior of the HostFigure and the stored Data in the HostModel
 * 
 * @author Daniel Meyer and Andre Schucht
 */
public class HostEditPart extends AbstractGraphicalEditPart implements
        PropertyChangeListener {

    @SuppressWarnings("unused")
    private List<Point> pointlist = new ArrayList<Point>();

    /**
     * host constraint change event
     */
    public static final String HOST_CONSTRAINT_CHANGED = "hostconstraintchanged"; //$NON-NLS-1$

    /**
     * Creates a org.eclipse.draw2d.IFigure
     * 
     * @return IFigure
     */
    @Override
    protected IFigure createFigure() {

        // Create Figure
        HostFigure figure1 = new HostFigure();

        //set name of figure
        figure1.setHost(((HostModel) getModel()).getName());
        
        //set the bounds
        figure1.setBounds(((HostModel) getModel()).getConstraint());
        
        //init size
        figure1.setSize( ((HostModel) getModel()).getConstraint().getSize());
       
        figure1.setBackgroundColor(VisualizerColors.BODY_1);
        figure1.setForegroundColor(VisualizerColors.HEAD_1);
        
        //set constraint of HostModel
        ((HostModel) getModel()).setConstraint(figure1.getBounds()
                .getCopy());

        //set the layoutmanager as XYLayout
        figure1.setLayoutManager(new XYLayout());

        return figure1;
    }

    /**
     * Returns the corresponding Models list of Children
     * 
     * @return list List of children
     */
    @Override
    protected List<ComponentModel> getModelChildren() {
        List<ComponentModel> list = ((HostModel) getModel()).getChildren();
        return list;
    }

    /**
     * @return the EditParts Model
     */
    public HostModel getHost() {
        return (HostModel) getModel();
    }

    /**
     * Installs EditPolicies to react on User-Interactions
     */
    public @Override void createEditPolicies() {
        installEditPolicy(EditPolicy.LAYOUT_ROLE,
                new MoveableComponentEditPolicy());
    }

    /**
     * Reacts on Changes made to the corresponding model
     * 
     * @param event
     *            PropertyChangeEvent fired by an HostModel
     * @see model.HostModel.firePropertyChange()
     */
    public void propertyChange(PropertyChangeEvent evt) {

    	if (evt.getPropertyName().equals("SCALE")){
    		
	    	if (evt.getSource() instanceof MoveCommand){
	    		
	    		ArrayList<Float> a = ((DiagramEditPart)getParent()).getMultiplier((Rectangle)evt.getOldValue(),(Rectangle)evt.getNewValue());
	    		
	            for (ComponentModel cm : getModelChildren()){
	                cm.firePropertyChange("SCALE",null,a); //$NON-NLS-1$
	            }
	    	}
	    	
	        ((DiagramEditPart)getParent()).propertyChange(new PropertyChangeEvent(
	        		this,"CHILDISH_CHANGE",null,null));
    	}
    	
        // Component deleted
        if (evt.getPropertyName().equals("COMPONENT_DELETE")) { //$NON-NLS-1$
            updateLayout((ComponentModel) evt.getOldValue(), evt.getNewValue(),
                    null);
        }
        
        if (evt.getPropertyName().equals("OUTPUT")) //$NON-NLS-1$
        {
            refreshSourceConnections();
            refreshTargetConnections();
        }
        //  New Connection as Target
        else if (evt.getPropertyName().equals("INPUT")) //$NON-NLS-1$
        {
            refreshTargetConnections();
            refreshSourceConnections();
        }

        // new Component or Constraint changed
        if (evt.getPropertyName().equals(HostModel.PROPERTY_NEW_COMPONENT)) {

            if (evt.getNewValue() instanceof ComponentEditPart){
                
            	removeChild((ComponentEditPart) evt.getNewValue());
            	while (children.contains((ComponentEditPart) evt.getNewValue())){
            		//wait for removal
            	}
            }
            refresh();
            calculateSize(getModelChildren());
        		refreshChildren();
          }

        if (evt.getPropertyName().equals(HostModel.PROPERTY_CONSTRAINT_CHANGED)){
        		getFigure().setBounds(((HostModel)getModel()).getConstraint());
            ((DiagramEditPart)getParent()).propertyChange(new PropertyChangeEvent(
            		this,"CHILDISH_CHANGE",null,null));
           }
        
        if (evt.getPropertyName().equals(ComponentEditPart.PROPERTY_CHILDISH_CHANGE)){
        	   calculateSize(getModelChildren());
        	   //      Causes Graph to re-layout
        		((GraphicalEditPart) (getViewer().getContents())).getFigure()
                .revalidate();
           }
                   
        refreshChildren();

        refreshVisuals();

    }

    /**
     * Calculates the Models Size by adding up the Childrens size
     * 
     * @param modelChildren
     *            List<ComponentModel>
     * @return The new Bounds
     */
    public Rectangle calculateSize(List<ComponentModel> modelChildren) {

        Rectangle bounds = new Rectangle(getFigure().getBounds().getLocation(),
        																	new Dimension(0,0));
        
        Point p = bounds.getLocation();
        
        Rectangle save = bounds.getCopy();

        if (modelChildren.isEmpty()) 
            bounds = new Rectangle(getFigure().getBounds().getLocation(),
					new Dimension(getFigure().getMinimumSize()));
        
        for (ComponentModel cm : modelChildren) {
        	
        		if (((ComponentModel)cm).getEditPart() == null) return null;

        		ComponentEditPart cep = (ComponentEditPart) ((ComponentModel)cm).getEditPart();

        		IFigure f = cep.getFigure();
        		bounds.union(f.getBounds());
            
           }
        
        bounds.setLocation(p);
        	//set new bounds of HostModel
        ((HostModel) getModel()).setConstraint(bounds);

        //give the childish change event to DiagramEditPart
        ((DiagramEditPart)getParent()).propertyChange(new PropertyChangeEvent(
        		this,"CHILDISH_CHANGE",save,bounds));
        
        return bounds;
    }

    /**
     * Activates this EditPart, used to register Listeners
     */
    public void activate() {
        ((HostModel) getModel()).addPropertyChangeListener(this);
        super.activate();
    }

    /**
     * Deactivates the EditPart, used to unregister Listeners
     */
    public void deactivate() {
        ((HostModel) getModel()).removePropertyChangeListener(this);
        super.deactivate();
    }

    /**
     * Repaints the HostFigure and his Children
     */
    protected void refreshVisuals() {
        Rectangle constraint = ((HostModel) getModel()).getConstraint();
        ((HostFigure)getFigure()).setHost(" "+((HostModel)getModel()).getName()+" ");
        ((GraphicalEditPart) getParent()).setLayoutConstraint(this,
                getFigure(), constraint);
        refreshChildren();
    }

    /**
     * Seeks on the view for a ponit to draw the new element (Host)
     * @param HostModel hm (Currently unused)
     * @return point to start painting the new element
     */
    public Point getAutomaticLocation(ComponentModel hm) {
    	
    	int minDist = 50;
    	
    	Point temp = new Point (0,0);
    	
    	int i =getModelChildren().size()+1;
    	    	
    	if (i==1) return temp;
    	
    	int z = (int)Math.ceil(Math.sqrt(i));
    	
    	int maxElements = z*z;
    	
    	if (i==maxElements){
    		//Rechteck komplett
    		Rectangle r = getModelChildren().get(i-2).getConstraint();
//    		Rectangle rabove = getModelChildren().get((z-1)*(z-1)-1).getConstraint();
    
//			if (r.y < (rabove.y+rabove.height)){
//				
//				for (int j=maxElements-z+1;maxElements>j;j++){
//					getModelChildren().get(j).setLocation(new Point(r.x,rabove.y+rabove.height+50));
//				}
//				r.y = rabove.y+rabove.height+minDist;
//			}
//			System.out.println("Unten Rechts: "+i+" "+z+" "+maxElements);
			return new Point(r.x+r.width+minDist,r.y);
    	}
    	else if (i <= maxElements-z){
    		if (i==((z-1)*(z-1)+1)){
    			//rechts oben
    			
    			Rectangle r = getModelChildren().get((z-2)*(z-2)).getConstraint();
    			temp.x = r.x+r.width+minDist;
//    			System.out.println("Rechts Oben: "+i+" "+z+" "+maxElements);
    			return temp;
    		}
    		else{
    			Rectangle r = getModelChildren().get(i-2).getConstraint();
    			temp.x = r.x;
    			temp.y = r.y+r.height+minDist;
//    			System.out.println("Rechts: "+i+" "+z+" "+maxElements);
    			return temp;
    		//nimm vorgänger und setz drunter
    		}
    	}
    	else if (i > maxElements-z+1){
    		//nimm vorgänger und setz daneben
    		Rectangle r = getModelChildren().get(i-2).getConstraint();
			temp.x = r.x+r.width+minDist;
			temp.y = r.y;
//			System.out.println("Unten: "+i+" "+z+" "+maxElements);
			return temp;
    	}
    	else if (i==maxElements-z+1){
    		//setze unten links unter (z-1)^2-z+1
    		Rectangle r = getModelChildren().get((z-2)*(z-2)+(z-2)).getConstraint();
			temp.x = r.x;
			temp.y = r.y+r.height+minDist;
//			System.out.println("Unten Links: "+i+" "+z+" "+maxElements);
			return temp;
    	}
//    	System.out.println("None: "+i+" "+z+" "+maxElements);
    	return null;
    }

    /**
     * starts the method ComponentModel findModelAt(Point temp) with the overgiven values
     * 
     * @param x position of point on x - axle
     * @param y position of point on y - axle
     * @return findModelAt
     */  
    public ComponentModel findModelAt(int x, int y) {
        return findModelAt(new Point(x, y));
    }
    
    /**
     * searches on the view for an other ComponentModel at the point temp
     * 
     * @param temp, the point that is possiply occured
     * @return Model found at point temp or null
     */
    public ComponentModel findModelAt(Point temp) {
        for (int i = 0; i < ((HostModel) getModel()).getChildren().size(); i++) {
            if (temp.x >= ((HostModel) getModel()).getChildren().get(i)
                    .getLocation().x
                    && temp.x <= ((HostModel) getModel()).getSize().width 
                            + ((HostModel) getModel()).getLocation().x
                    && temp.y >= ((HostModel) getModel()).getLocation().y
                    && temp.y <= ((HostModel) getModel()).getSize().height 
                            + ((HostModel) getModel()).getLocation().y
            ) 
                return ((HostModel) getModel()).getChildren().get(i);
            
        }
        return null;
    }

    /**
     * Updates the Pointlist (Currently unused)
     * 
     * @param model
     * @param object
     * @param object2
     */
    private void updateLayout(@SuppressWarnings("unused")
    ComponentModel model, @SuppressWarnings("unused")
    Object object, @SuppressWarnings("unused")
    Object object2) {

        // Update the Pointlist...

        // Update the View
        ((GraphicalEditPart) (getViewer().getContents())).getFigure()
                .revalidate();
    }
    
    /**************************Connections*********************/

    /**
     * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#getModelSourceConnections()
     */
    @Override
    protected List getModelSourceConnections()
    {
        return getHost().getStartConnections();
    }
    
    /**
     * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#getModelTargetConnections()
     */
    @Override
    protected List getModelTargetConnections()
    {
        return getHost().getEndConnections();
    }

	public boolean checkCollision(ComponentModel aa, ComponentModel oo) {
		// TODO Auto-generated method stub
		return false;
	}
}