/*
 * Copyright (C) 2006 Johannes Baumgarten, Bjoern Gebhardt, Mathias Kellner, Stephan Kluge, Svetlana Li, Jurij Merker, Daniel Meyer, Andre Schucht, Ling Wu
 *
 * This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 2 of the License. 
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details. 
 * You should have received a copy of the GNU General Public License along with this program; if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA
 */

package Debug.Visualizer.editparts;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.draw2d.*;
import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.draw2d.geometry.Rectangle;

import org.eclipse.gef.GraphicalEditPart;
import org.eclipse.gef.editparts.AbstractGraphicalEditPart;

import Debug.Visualizer.figure.InfoscreenFigure;
import Debug.Visualizer.gui.VisualizerColors;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import Debug.Visualizer.model.*;

/**
 * Controls the behavior of the InfoscreenFigure and the stored Data in 
 * the InfoscreenModel
 * @author Daniel Meyer 
 */
public class InfoscreenEditPart extends AbstractGraphicalEditPart 
	implements PropertyChangeListener{

	//USED FOR PARENT TO RESIZE
	public static final String INFOSCREEN_CONSTRAINT_CHANGED = "Infoscreen_Constraint_changed"; //$NON-NLS-1$
	
	@SuppressWarnings("unused")
    private List<Point> pointlist = new ArrayList<Point>();
	
	/**
	 * Creates a org.eclipse.draw2d.IFigure
	 * @return IFigure  
	 */
	@Override
	protected IFigure createFigure() {
	
		//create figure
		InfoscreenFigure figure1 = new InfoscreenFigure(((InfoscreenModel)getModel()).getName());
		
		//set bounds of figure
		figure1.setBounds(((InfoscreenModel)getModel()).getConstraint());
		
		figure1.setBackgroundColor(VisualizerColors.BODY_4);
		figure1.setForegroundColor(VisualizerColors.HEAD_2);
		
		//calculate maxwidth of figure
      InfoscreenDataEditPart.maxwidth = figure1.getBounds().width - figure1.getBorder().getInsets(figure1).getWidth();
        
      //set layoutmanager of figure as XYlayout
      figure1.setLayoutManager(new XYLayout());
        
		return figure1;
	}
	
	/**
	 * Returns the corresponding Models list of Children
	 * @return list List of children 
	 */
	@Override
	protected List<InfoscreenDataModel> getModelChildren() {
		return ((InfoscreenModel)getModel()).getChildren();
	}
	
	/**
	 * Installs EditPolicies to react on User-Interactions
	 */
	@Override
	public void createEditPolicies() {
		//Dummy
	}
	
	/**
	 * Reacts on Changes made to the corresponding model
	 * @param event PropertyChangeEvent fired by an InfoscreenModel
	 * @see model.InfoscreenModel.firePropertyChange()
	 */
	public void propertyChange(PropertyChangeEvent event) {
		
		//MODEL SCALED
		if (event.getPropertyName().equals("SCALE")){
	    		
	    		ArrayList<Float> a = ((DiagramEditPart)getParent()).getMultiplier((Rectangle)event.getOldValue(),(Rectangle)event.getNewValue());
	    		
	    		InfoscreenDataEditPart.submaxwidth = (int)Math.ceil(InfoscreenDataEditPart.submaxwidth*a.get(0));
	    		InfoscreenDataEditPart.txtmaxwidth = (int)Math.ceil(InfoscreenDataEditPart.txtmaxwidth*a.get(0));
	    		InfoscreenDataEditPart.maxheight   = (int)Math.ceil(InfoscreenDataEditPart.maxheight*a.get(1));
	    		
	            for (InfoscreenDataModel dem : getModelChildren()){
	                dem.firePropertyChange("SCALE",null,a);
	            }
	            
	        	refreshChildrenSize(getModelChildren());
	        	refreshVisuals();
	        	refreshChildren();
	            ((DiagramEditPart)getParent()).propertyChange(new PropertyChangeEvent(
	            		this,"CHILDISH_CHANGE",null,null));
	        	
    	}
		
        // Model RESIZED
        if (event.getPropertyName().equals(InfoscreenModel.INFOSCREEN_CONSTRAINT_CHANGED)) {

        	Rectangle r = (Rectangle)event.getNewValue();
        	
            getFigure().setBounds(r);
            
            InfoscreenDataEditPart.maxwidth = r.width-getFigure().getBorder().getInsets(getFigure()).getWidth();
            
        	refreshChildrenSize(getModelChildren());
        	refreshVisuals();
        	refreshChildren();
        	
        	((DiagramEditPart)getParent()).propertyChange(new PropertyChangeEvent(this,"CHILDISH_CHANGE",null,null));
        }
		
        // InfoscreenDataModel added or Deleted
        else if (event.getPropertyName().equals(InfoscreenModel.NEWDATA)){
            
        	refreshChildrenSize(getModelChildren());
        	refreshVisuals();
        	refreshChildren();
        }
        
        //NEW NAME ADDON
        else if (event.getPropertyName().equals(InfoscreenModel.INFOSCREEN_NAME_CHANGED)){
			refreshVisuals();
		}
	}
	
    /**
     * Reszize the Children by the Parent Models Size
     * @param modelChildren List<InfoscreenDataModel>
     */
	public void refreshChildrenSize(List<InfoscreenDataModel> modelChildren) {
		
			Rectangle parent = getFigure().getClientArea();
			int i=0;

			for (InfoscreenDataModel idm : modelChildren){
				
				Rectangle r = new Rectangle(parent.x,parent.y+(idm.getSize().height*i),
						InfoscreenDataEditPart.maxwidth,idm.getSize().height);
				
				idm.setConstraint(r);
				
				i++;
			}
	}

	/**
	 * Activates this EditPart, used to register Listeners
	 */
	@Override
	public void activate() {
		((InfoscreenModel) getModel()).addPropertyChangeListener(this);
		super.activate();
	}
	
	/**
	 * Deactivates the EditPart, used to unregister Listeners
	 */
	@Override
	public void deactivate() {
		((InfoscreenModel) getModel()).removePropertyChangeListener(this);
		super.deactivate();
	}
	
	/**
	 * Repaints the InfoscreenFigure and its Children
	 */
	@Override
	protected void refreshVisuals() {
		
		Rectangle constraint = ((InfoscreenModel) getModel()).getConstraint();
		
        ((AbstractLabeledBorder)((CompoundBorder)((CompoundBorder) 
        		((InfoscreenFigure)getFigure() ).getBorder()).getInnerBorder())
        		.getInnerBorder()).setLabel(("Infoscreen"+((InfoscreenModel)
        		getModel()).getName()));
        
        ((InfoscreenFigure)getFigure() ).repaint();
        
		((GraphicalEditPart) getParent()).setLayoutConstraint(
			this,
			getFigure(),
			constraint);
		
		refreshChildren();
	}
	
    /**
     * Seeks on the view for a point to draw the new element
     * 
     * @return calculated point to start painting the new element
     */
	public Point getAutomaticLocation() {
		
		Point x = new Point(((InfoscreenModel)getModel()).getLocation());
		
		int y = getModelChildren().size()-1;
		
		int z = InfoscreenDataEditPart.maxheight;

		x.y+=(y*z);
        
        if (((y*z) + getFigure().getBorder().getInsets(getFigure()).getHeight() ) > ((InfoscreenModel)getModel()).getSize().height)
            ((InfoscreenModel)getModel()).setSize(new Dimension(
                    ((InfoscreenModel)getModel()).getSize().width,
                    z*y+getFigure().getBorder().getInsets(getFigure()).getHeight()));
		
		return x;
	}

	/**
	 * Resizes the Models Children
	 * @see resizeChildren(List<InfoscreenDataModel>)
	 */
	public void refreshChildrenSize() {
		refreshChildrenSize(getModelChildren());
	}
}